# coding: utf-8

from otcbtc_client.kline import Kline
from otcbtc_client.market import Market
from otcbtc_client.order_book import OrderBook
from otcbtc_client.ticker import Ticker
from otcbtc_client.timestamp import Timestamp
from otcbtc_client.trade import Trade
from otcbtc_client.user import User
from otcbtc_client.order import Order


class OTCBTCClient(object):
    def __init__(self, api_key='', api_secret=''):
        self.api_key = api_key
        self.api_secret = api_secret

    @property
    def market(self):
        return Market()

    @property
    def ticker(self):
        return Ticker()

    @property
    def order_book(self):
        return OrderBook()

    @property
    def trade(self):
        return Trade(self.api_key, self.api_secret)

    @property
    def timestamp(self):
        return Timestamp()

    @property
    def kline(self):
        return Kline()

    @property
    def user(self):
        return User(self.api_key, self.api_secret)

    @property
    def order(self):
        return Order(self.api_key, self.api_secret)


# coding: utf-8

from otcbtc_client.base import OTCBTCAPIBase

class Timestamp(OTCBTCAPIBase):

    def fetch(self):
        return self.get('/api/v2/timestamp')

    def __call__(self):
        return self.fetch()


# coding: utf-8

from otcbtc_client.base import OTCBTCAPIBase

class Kline(OTCBTCAPIBase):

    def fetch(self, market, limit=30, period=1, timestamp=None):
        params =  {
            'market': market,
            'limit': limit,
            'period': period
        }
        if timestamp:
            params.update(timestamp=timestamp)
        return self.get('/api/v2/klines', params=params)

    def with_pending_trades(self, market, trade_id, limit=30, period=1, timestamp=None):
        params =  {
            'market': market,
            'trade_id': trade_id,
            'limit': limit,
            'period': period
        }
        if timestamp:
            params.update(timestamp=timestamp)
        return self.get('/api/v2/klines_with_pending_trades', params=params)


# coding: utf-8

import hashlib
import hmac
from builtins import bytes

import requests


class OTCBTCAPIBase(object):

    API_ENDPOINT = 'https://bb.otcbtc.com'

    def __init__(self, api_key='', api_secret=''):
        self.access_key = api_key
        self.api_secret = api_secret

    def build_url(self, uri):
        return self.API_ENDPOINT + uri

    def build_payload(self, method, uri, params):
        # parameters concated with "&", MUST BE IN ALPHABETICAL ORDER by parameters
        params = '&'.join(['%s=%s' % (k, v) for k, v in sorted(list(params.items()), key=lambda t: t[0])])
        # payload is a string represents this request, combiled with HTTP method, request URI and request parameters
        return '|'.join([method, uri, params])

    def generate_signature(self, payload):
        # signature: can be generated by HMAC-SHA256(payload, your_api_secret).to_hex
        return hmac.new(bytes(self.api_secret, 'ascii'), bytes(payload, 'ascii'), hashlib.sha256).hexdigest()

    def get(self, uri, params=None, **kwargs):
        r = requests.get(self.build_url(uri), params=params, **kwargs)
        return r.json()

    def post(self, uri, params=None, **kwargs):
        r = requests.post(self.build_url(uri), params=params, **kwargs)
        return r.json()



# coding: utf-8

from otcbtc_client.base import OTCBTCAPIBase


class Ticker(OTCBTCAPIBase):

    URI = '/api/v2/tickers'

    def all(self):
        return self.get(self.URI)

    def fetch(self, market_id):
        return self.get('{uri}/{market_id}'.format(
            uri=self.URI, market_id=market_id))



# coding: utf-8

from otcbtc_client.base import OTCBTCAPIBase


class Trade(OTCBTCAPIBase):

    TRADES_URI = '/api/v2/trades'
    MY_TRADES_URI = '/api/v2/trades/my'

    def fetch(self,
              market,
              limit=50,
              timestamp=None,
              from_=None,
              to=None,
              order_by='desc'):
        params = {
            k: v
            for k, v in {
                'market': market,
                'limit': limit,
                'timestamp': timestamp,
                'from': from_,
                'to': to,
                'order_by': order_by
            }.items() if v
        }
        return self.get(self.TRADES_URI, params=params)

    def my_trades(self,
                  market,
                  limit=None,
                  timestamp=None,
                  from_=None,
                  to=None,
                  order_by=None):
        uri = self.MY_TRADES_URI
        params = {
            k: v
            for k, v in {
                'access_key': self.access_key,
                'market': market,
                'limit': limit,
                'timestamp': timestamp,
                'from': from_,
                'to': to,
                'order_by': order_by
            }.items() if v
        }
        payload = self.build_payload('GET', uri, params)
        params.update(signature=self.generate_signature(payload))
        return self.get(uri, params=params)


# coding:  utf-8

from otcbtc_client.base import OTCBTCAPIBase

class Order(OTCBTCAPIBase):

    def list_order(self, id):
        uri = '/api/v2/order'
        params = {
            'access_key': self.access_key,
            'id': id
        }
        payload = self.build_payload('GET', uri, params)
        params.update(signature=self.generate_signature(payload))
        return self.get(uri, params=params)

    def list_orders(self, market=None, state='wait', limit=100, page=1, order_by='asc'):
        # state: Filter order by state, default to ‘wait’ (active orders). Other options:‘cancel’, ‘done’
        uri = '/api/v2/orders'
        params = {
            'access_key': self.access_key,
            'state': state,
            'limit': limit,
            'page': page,
            'order_by': order_by
        }
        if market:
            params.update(market=market)
        payload = self.build_payload('GET', uri, params)
        params.update(signature=self.generate_signature(payload))
        return self.get(uri, params=params)

    def create_order(self, market, side, volume, price, ord_type='limit'):
        uri = '/api/v2/orders'
        params = {
            'access_key': self.access_key,
            'market': market,
            'side': side,
            'volume': volume,
            'price': price,
            'ord_type': ord_type
        }
        payload = self.build_payload('POST', uri, params)
        params.update(signature=self.generate_signature(payload))
        return self.post(uri, params=params)

    def cancel_order(self, id):
        uri = '/api/v2/order/delete'
        params = {
            'access_key': self.access_key,
            'id': id
        }
        payload = self.build_payload('POST', uri, params)
        params.update(signature=self.generate_signature(payload))
        return self.post(uri, params=params)

    def cancel_orders(self):
        uri = '/api/v2/orders/clear'
        params = {
            'access_key': self.access_key,
        }
        payload = self.build_payload('POST', uri, params)
        params.update(signature=self.generate_signature(payload))
        return self.post(uri, params=params)


# coding: utf-8

from otcbtc_client.base import OTCBTCAPIBase

class User(OTCBTCAPIBase):

    def fetch(self):
        uri = '/api/v2/users/me'
        params = {
            'access_key': self.access_key,
        }
        payload = self.build_payload('GET', uri, params=params)
        params.update(signature=self.generate_signature(payload))
        return self.get(uri, params=params)


import setuptools

setuptools.setup(
    name='otcbtc_client',
    version='0.1.0',
    author='Gimo',
    author_email='self@gimo.me',
    description='OTCBTC SDK for Python',
    url='https://github.com/masakichi/otcbtc-client',
    packages=setuptools.find_packages(exclude=['tests']),
    classifiers=('Intended Audience :: Developers',
                 'License :: OSI Approved :: MIT License',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python :: 2',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3',
                 'Programming Language :: Python :: 3.4',
                 'Programming Language :: Python :: 3.5',
                 'Programming Language :: Python :: 3.6',
                 'Programming Language :: Python :: Implementation :: CPython',
                 'Programming Language :: Python :: Implementation :: PyPy'),
    install_requires=[
        'requests >=2.11, <3.0a0',
        'future',
    ])



# coding: utf-8

from otcbtc_client.base import OTCBTCAPIBase


class OrderBook(OTCBTCAPIBase):

    URI = '/api/v2/order_book'

    def fetch(self, market, asks_limit=20, bids_limit=20):
        return self.get(
            self.URI,
            params={
                'market': market,
                'asks_limit': asks_limit,
                'bids_limit': bids_limit
            })
